@@ -3,47 +3,71 @@
 
 const char* sqlite_code_string(int code);
 const char* sqlite_authorizer_string(int type);
+#include <vector>
 
+// TODO: better way to work around StringConcat?
+#include <napi.h>
+inline Napi::String StringConcat(Napi::Value str1, Napi::Value str2) {
+  return Napi::String::New(str1.Env(), str1.As<Napi::String>().Utf8Value() +
+                    str2.As<Napi::String>().Utf8Value() );
+}
+
+// A Napi substitute IsInt32()
+inline bool OtherIsInt(Napi::Number source) {
+    double orig_val = source.DoubleValue();
+    double int_val = (double)source.Int32Value();
+    if (orig_val == int_val) {
+        return true;
+    } else {
+        return false;
+    }
+}
 
 #define REQUIRE_ARGUMENTS(n)                                                   \
     if (info.Length() < (n)) {                                                 \
-        return Nan::ThrowTypeError("Expected " #n "arguments");                \
+        Napi::TypeError::New(env, "Expected " #n "arguments").ThrowAsJavaScriptException(); \
+        return env.Null(); \
     }
 
 
 #define REQUIRE_ARGUMENT_EXTERNAL(i, var)                                      \
-    if (info.Length() <= (i) || !info[i]->IsExternal()) {                      \
-        return Nan::ThrowTypeError("Argument " #i " invalid");                 \
+    if (info.Length() <= (i) || !info[i].IsExternal()) {                       \
+        Napi::TypeError::New(env, "Argument " #i " invalid").ThrowAsJavaScriptException(); \
+        return env.Null(); \
     }                                                                          \
-    Local<External> var = Local<External>::Cast(info[i]);
+    Napi::External var = info[i].As<Napi::External>();
 
 
 #define REQUIRE_ARGUMENT_FUNCTION(i, var)                                      \
-    if (info.Length() <= (i) || !info[i]->IsFunction()) {                      \
-        return Nan::ThrowTypeError("Argument " #i " must be a function");      \
+    if (info.Length() <= (i) || !info[i].IsFunction()) {                        \
+        Napi::TypeError::New(env, "Argument " #i " must be a function").ThrowAsJavaScriptException(); \
+        return env.Null(); \
     }                                                                          \
-    Local<Function> var = Local<Function>::Cast(info[i]);
+    Napi::Function var = info[i].As<Napi::Function>();
 
 
 #define REQUIRE_ARGUMENT_STRING(i, var)                                        \
-    if (info.Length() <= (i) || !info[i]->IsString()) {                        \
-        return Nan::ThrowTypeError("Argument " #i " must be a string");        \
+    if (info.Length() <= (i) || !info[i].IsString()) {                         \
+        Napi::TypeError::New(env, "Argument " #i " must be a string").ThrowAsJavaScriptException(); \
+        return env.Null(); \
     }                                                                          \
-    Nan::Utf8String var(info[i]);
+    std::string var = info[i].As<Napi::String>();
 
 #define REQUIRE_ARGUMENT_INTEGER(i, var)                                        \
-    if (info.Length() <= (i) || !info[i]->IsInt32()) {                        \
-        return Nan::ThrowTypeError("Argument " #i " must be an integer");        \
+    if (info.Length() <= (i) || !info[i].IsNumber()) {                        \
+        Napi::TypeError::New(env, "Argument " #i " must be an integer").ThrowAsJavaScriptException(); \
+        return env.Null();        \
     }                                                                          \
-    int var(Nan::To<int32_t>(info[i]).FromJust());
+    int var(info[i].As<Napi::Number>().Int32Value());
 
 #define OPTIONAL_ARGUMENT_FUNCTION(i, var)                                     \
-    Local<Function> var;                                                       \
-    if (info.Length() > i && !info[i]->IsUndefined()) {                        \
-        if (!info[i]->IsFunction()) {                                          \
-            return Nan::ThrowTypeError("Argument " #i " must be a function");  \
+    Napi::Function var;                                                        \
+    if (info.Length() > i && !info[i].IsUndefined()) {                         \
+        if (!info[i].IsFunction()) {                                           \
+            Napi::TypeError::New(env, "Argument " #i " must be a function").ThrowAsJavaScriptException(); \
+            return env.Null(); \
         }                                                                      \
-        var = Local<Function>::Cast(info[i]);                                  \
+        var = info[i].As<Napi::Function>();                                    \
     }
 
 
@@ -52,70 +76,62 @@ const char* sqlite_authorizer_string(int type);
     if (info.Length() <= (i)) {                                                \
         var = (default);                                                       \
     }                                                                          \
-    else if (info[i]->IsInt32()) {                                             \
-        var = Nan::To<int32_t>(info[i]).FromJust();                            \
+    else if (info[i].IsNumber()) {                                             \
+        if (OtherIsInt(info[i].As<Number>())) {                                \
+            var = info[i].As<Napi::Number>().Int32Value();                     \
+        }                                                                      \
     }                                                                          \
     else {                                                                     \
-        return Nan::ThrowTypeError("Argument " #i " must be an integer");      \
+        Napi::TypeError::New(env, "Argument " #i " must be an integer").ThrowAsJavaScriptException(); \
+        return env.Null(); \
     }
 
 
 #define DEFINE_CONSTANT_INTEGER(target, constant, name)                        \
-    Nan::ForceSet(target,                                                      \
-        Nan::New(#name).ToLocalChecked(),                                      \
-        Nan::New<Integer>(constant),                                           \
-        static_cast<PropertyAttribute>(ReadOnly | DontDelete)                  \
-    );
+    Napi::PropertyDescriptor::Value(#name, Napi::Number::New(env, constant),   \
+        static_cast<napi_property_attributes>(napi_enumerable | napi_configurable)),
 
 #define DEFINE_CONSTANT_STRING(target, constant, name)                         \
-    Nan::ForceSet(target,                                                      \
-        Nan::New(#name).ToLocalChecked(),                                      \
-        Nan::New(constant).ToLocalChecked(),                                   \
-        static_cast<PropertyAttribute>(ReadOnly | DontDelete)                  \
-    );
-
-
-#define NODE_SET_GETTER(target, name, function)                                \
-    Nan::SetAccessor((target)->InstanceTemplate(),                             \
-        Nan::New(name).ToLocalChecked(), (function));
-
-#define NODE_SET_SETTER(target, name, getter, setter)                          \
-    Nan::SetAccessor((target)->InstanceTemplate(),                             \
-        Nan::New(name).ToLocalChecked(), getter, setter);
-
-#define GET_STRING(source, name, property)                                     \
-    Nan::Utf8String name(Nan::Get(source,                                      \
-        Nan::New(prop).ToLocalChecked()).ToLocalChecked());
-
-#define GET_INTEGER(source, name, prop)                                        \
-    int name = Nan::To<int>(Nan::Get(source,                                   \
-        Nan::New(property).ToLocalChecked()).ToLocalChecked()).FromJust();
+    Napi::PropertyDescriptor::Value(#name, Napi::String::New(env, constant),   \
+        static_cast<napi_property_attributes>(napi_enumerable | napi_configurable)),
 
 #define EXCEPTION(msg, errno, name)                                            \
-    Local<Value> name = Exception::Error(Nan::New(                             \
-        std::string(sqlite_code_string(errno)) +                               \
-        std::string(": ") + std::string(msg)                                   \
-    ).ToLocalChecked());                                                                        \
-    Local<Object> name ##_obj = name.As<Object>();                             \
-    Nan::Set(name ##_obj, Nan::New("errno").ToLocalChecked(), Nan::New(errno));\
-    Nan::Set(name ##_obj, Nan::New("code").ToLocalChecked(),                   \
-        Nan::New(sqlite_code_string(errno)).ToLocalChecked());
+    Napi::Value name = Napi::Error::New(env,                                   \
+        StringConcat(                                                          \
+            StringConcat(                                                      \
+                Napi::String::New(env, sqlite_code_string(errno)),             \
+                Napi::String::New(env, ": ")                                   \
+            ),                                                                 \
+            (msg)                                                              \
+        ).Utf8Value()                                                          \
+    ).Value();                                                                 \
+    Napi::Object name ##_obj = name.As<Napi::Object>();                        \
+    (name ##_obj).Set( Napi::String::New(env, "errno"), Napi::Number::New(env, errno)); \
+    (name ##_obj).Set( Napi::String::New(env, "code"),                         \
+        Napi::String::New(env, sqlite_code_string(errno)));
+
 
 #define EMIT_EVENT(obj, argc, argv)                                            \
     TRY_CATCH_CALL((obj),                                                      \
-        Nan::Get(obj,                                                          \
-            Nan::New("emit").ToLocalChecked()).ToLocalChecked().As<Function>(),\
+        (obj).Get("emit").As<Napi::Function>(),\
         argc, argv                                                             \
     );
 
+// The Mac OS compiler complains when argv is NULL unless we
+// first assign it to a locally defined variable.
 #define TRY_CATCH_CALL(context, callback, argc, argv)                          \
-    Nan::MakeCallback((context), (callback), (argc), (argv))
+    Napi::Value* passed_argv = argv;\
+    std::vector<napi_value> args;\
+    if ((argc != 0) && (passed_argv != NULL)) {\
+      args.assign(passed_argv, passed_argv + argc);\
+    }\
+    (callback).MakeCallback(Napi::Value(context), args);
 
 #define WORK_DEFINITION(name)                                                  \
-    static NAN_METHOD(name);                                                   \
+    Napi::Value name(const Napi::CallbackInfo& info);                          \
     static void Work_Begin##name(Baton* baton);                                \
-    static void Work_##name(uv_work_t* req);                                   \
-    static void Work_After##name(uv_work_t* req);
+    static void Work_##name(napi_env env, void* data);                         \
+    static void Work_After##name(napi_env env, napi_status status, void* data);
 
 #define STATEMENT_BEGIN(type)                                                  \
     assert(baton);                                                             \
@@ -125,13 +141,16 @@ const char* sqlite_authorizer_string(int type);
     assert(baton->stmt->prepared);                                             \
     baton->stmt->locked = true;                                                \
     baton->stmt->db->pending++;                                                \
-    int status = uv_queue_work(uv_default_loop(),                              \
-        &baton->request,                                                       \
-        Work_##type, reinterpret_cast<uv_after_work_cb>(Work_After##type));    \
-    assert(status == 0);
+    Napi::Env env = baton->stmt->Env();                                        \
+    int status = napi_create_async_work(                                       \
+        env, NULL, Napi::String::New(env, "sqlite3.Statement."#type),          \
+        Work_##type, Work_After##type, baton, &baton->request                  \
+    );                                                                         \
+    assert(status == 0);                                                       \
+    napi_queue_async_work(env, baton->request);
 
 #define STATEMENT_INIT(type)                                                   \
-    type* baton = static_cast<type*>(req->data);                               \
+    type* baton = static_cast<type*>(data);                                    \
     Statement* stmt = baton->stmt;
 
 #define STATEMENT_END()                                                        \
@@ -141,6 +160,7 @@ const char* sqlite_authorizer_string(int type);
     stmt->db->pending--;                                                       \
     stmt->Process();                                                           \
     stmt->db->Process();                                                       \
+    napi_delete_async_work(e, baton->request);                                 \
     delete baton;
 
 #define BACKUP_BEGIN(type)                                                     \
@@ -151,13 +171,16 @@ const char* sqlite_authorizer_string(int type);
     assert(baton->backup->inited);                                             \
     baton->backup->locked = true;                                              \
     baton->backup->db->pending++;                                              \
-    int status = uv_queue_work(uv_default_loop(),                              \
-        &baton->request,                                                       \
-        Work_##type, reinterpret_cast<uv_after_work_cb>(Work_After##type));    \
-    assert(status == 0);
+    Napi::Env env = baton->backup->Env();                                      \
+    int status = napi_create_async_work(                                       \
+        env, NULL, Napi::String::New(env, "sqlite3.Backup."#type),             \
+        Work_##type, Work_After##type, baton, &baton->request                  \
+    );                                                                         \
+    assert(status == 0);                                                       \
+    napi_queue_async_work(env, baton->request);
 
 #define BACKUP_INIT(type)                                                      \
-    type* baton = static_cast<type*>(req->data);                               \
+    type* baton = static_cast<type*>(data);                                    \
     Backup* backup = baton->backup;
 
 #define BACKUP_END()                                                           \
@@ -167,6 +190,7 @@ const char* sqlite_authorizer_string(int type);
     backup->db->pending--;                                                     \
     backup->Process();                                                         \
     backup->db->Process();                                                     \
+    napi_delete_async_work(e, baton->request);                                 \
     delete baton;
 
 #define DELETE_FIELD(field)                                                    \
