@@ -1,7 +1,6 @@
 #include <string.h>
-#include <node.h>
-#include <node_buffer.h>
-#include <node_version.h>
+#include <napi.h>
+#include <uv.h>
 
 #include "macros.h"
 #include "database.h"
@@ -9,27 +8,37 @@
 
 using namespace node_sqlite3;
 
-Nan::Persistent<FunctionTemplate> Statement::constructor_template;
+Napi::FunctionReference Statement::constructor;
 
-NAN_MODULE_INIT(Statement::Init) {
-    Nan::HandleScope scope;
+Napi::Object Statement::Init(Napi::Env env, Napi::Object exports) {
+    Napi::HandleScope scope(env);
 
-    Local<FunctionTemplate> t = Nan::New<FunctionTemplate>(New);
+    Napi::Function t = DefineClass(env, "Statement", {
+      InstanceMethod("bind", &Statement::Bind),
+      InstanceMethod("get", &Statement::Get),
+      InstanceMethod("run", &Statement::Run),
+      InstanceMethod("all", &Statement::All),
+      InstanceMethod("each", &Statement::Each),
+      InstanceMethod("reset", &Statement::Reset),
+      InstanceMethod("finalize", &Statement::Finalize_),
+    });
 
-    t->InstanceTemplate()->SetInternalFieldCount(1);
-    t->SetClassName(Nan::New("Statement").ToLocalChecked());
+    constructor = Napi::Persistent(t);
+    constructor.SuppressDestruct();
 
-    Nan::SetPrototypeMethod(t, "bind", Bind);
-    Nan::SetPrototypeMethod(t, "get", Get);
-    Nan::SetPrototypeMethod(t, "run", Run);
-    Nan::SetPrototypeMethod(t, "all", All);
-    Nan::SetPrototypeMethod(t, "each", Each);
-    Nan::SetPrototypeMethod(t, "reset", Reset);
-    Nan::SetPrototypeMethod(t, "finalize", Finalize);
+    exports.Set("Statement", t);
+    return exports;
+}
 
-    constructor_template.Reset(t);
-    Nan::Set(target, Nan::New("Statement").ToLocalChecked(),
-        Nan::GetFunction(t).ToLocalChecked());
+// A Napi InstanceOf for Javascript Objects "Date" and "RegExp"
+bool OtherInstanceOf(Napi::Object source, char* object_type) {
+    if (object_type == "Date") {
+        return source.InstanceOf(source.Env().Global().Get("Date").As<Function>());
+    } else if (object_type == "RegExp") {
+        return source.InstanceOf(source.Env().Global().Get("RegExp").As<Function>());
+    }
+
+    return false;
 }
 
 void Statement::Process() {
@@ -60,67 +69,71 @@ void Statement::Schedule(Work_Callback callback, Baton* baton) {
 }
 
 template <class T> void Statement::Error(T* baton) {
-    Nan::HandleScope scope;
-
     Statement* stmt = baton->stmt;
+
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     // Fail hard on logic errors.
     assert(stmt->status != 0);
-    EXCEPTION(stmt->message, stmt->status, exception);
+    EXCEPTION(Napi::String::New(env, stmt->message.c_str()), stmt->status, exception);
 
-    Local<Function> cb = Nan::New(baton->callback);
+    Napi::Function cb = baton->callback.Value();
 
-    if (!cb.IsEmpty() && cb->IsFunction()) {
-        Local<Value> argv[] = { exception };
-        TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+    if (!cb.IsUndefined() && cb.IsFunction()) {
+        Napi::Value argv[] = { exception };
+        TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
     }
     else {
-        Local<Value> argv[] = { Nan::New("error").ToLocalChecked(), exception };
-        EMIT_EVENT(stmt->handle(), 2, argv);
+        Napi::Value argv[] = { Napi::String::New(env, "error"), exception };
+        EMIT_EVENT(stmt->Value(), 2, argv);
     }
 }
 
 // { Database db, String sql, Array params, Function callback }
-NAN_METHOD(Statement::New) {
-    if (!info.IsConstructCall()) {
-        return Nan::ThrowTypeError("Use the new operator to create new Statement objects");
-    }
-
+Statement::Statement(const Napi::CallbackInfo& info) : Napi::ObjectWrap<Statement>(info) {
+    Napi::Env env = info.Env();
     int length = info.Length();
 
     if (length <= 0 || !Database::HasInstance(info[0])) {
-        return Nan::ThrowTypeError("Database object expected");
+        Napi::TypeError::New(env, "Database object expected").ThrowAsJavaScriptException();
+        return;
     }
-    else if (length <= 1 || !info[1]->IsString()) {
-        return Nan::ThrowTypeError("SQL query expected");
+    else if (length <= 1 || !info[1].IsString()) {
+        Napi::TypeError::New(env, "SQL query expected").ThrowAsJavaScriptException();
+        return;
     }
-    else if (length > 2 && !info[2]->IsUndefined() && !info[2]->IsFunction()) {
-        return Nan::ThrowTypeError("Callback expected");
+    else if (length > 2 && !info[2].IsUndefined() && !info[2].IsFunction()) {
+        Napi::TypeError::New(env, "Callback expected").ThrowAsJavaScriptException();
+        return;
     }
 
-    Database* db = Nan::ObjectWrap::Unwrap<Database>(info[0].As<Object>());
-    Local<String> sql = Local<String>::Cast(info[1]);
+    Database* db = Napi::ObjectWrap<Database>::Unwrap(info[0].As<Napi::Object>());
+    Napi::String sql = info[1].As<Napi::String>();
 
-    Nan::ForceSet(info.This(),Nan::New("sql").ToLocalChecked(), sql, ReadOnly);
+    info.This().As<Napi::Object>().DefineProperty(Napi::PropertyDescriptor::Value("sql", sql, napi_default));
 
-    Statement* stmt = new Statement(db);
-    stmt->Wrap(info.This());
+    init(db);
+    Statement* stmt = this;
 
-    PrepareBaton* baton = new PrepareBaton(db, Local<Function>::Cast(info[2]), stmt);
-    baton->sql = std::string(*Nan::Utf8String(sql));
+    PrepareBaton* baton = new PrepareBaton(db, info[2].As<Napi::Function>(), stmt);
+    baton->sql = std::string(sql.As<Napi::String>().Utf8Value().c_str());
     db->Schedule(Work_BeginPrepare, baton);
-
-    info.GetReturnValue().Set(info.This());
 }
 
 void Statement::Work_BeginPrepare(Database::Baton* baton) {
     assert(baton->db->open);
     baton->db->pending++;
-    int status = uv_queue_work(uv_default_loop(),
-        &baton->request, Work_Prepare, (uv_after_work_cb)Work_AfterPrepare);
+    Napi::Env env = baton->db->Env();
+    int status = napi_create_async_work(
+        env, NULL, Napi::String::New(env, "sqlite3.Statement.Prepare"),
+        Work_Prepare, Work_AfterPrepare, baton, &baton->request
+    );
     assert(status == 0);
+    napi_queue_async_work(env, baton->request);
 }
 
-void Statement::Work_Prepare(uv_work_t* req) {
+void Statement::Work_Prepare(napi_env e, void* data) {
     STATEMENT_INIT(PrepareBaton);
 
     // In case preparing fails, we use a mutex to make sure we get the associated
@@ -144,21 +157,22 @@ void Statement::Work_Prepare(uv_work_t* req) {
     sqlite3_mutex_leave(mtx);
 }
 
-void Statement::Work_AfterPrepare(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterPrepare(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(PrepareBaton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_OK) {
         Error(baton);
-        stmt->Finalize();
+        stmt->Finalize_();
     }
     else {
         stmt->prepared = true;
-        Local<Function> cb = Nan::New(baton->callback);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
-            Local<Value> argv[] = { Nan::Null() };
-            TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+        if (!baton->callback.IsEmpty() && baton->callback.Value().IsFunction()) {
+            Napi::Function cb = baton->callback.Value();
+            Napi::Value argv[] = { env.Null() };
+            TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
         }
     }
 
@@ -166,77 +180,88 @@ void Statement::Work_AfterPrepare(uv_work_t* req) {
 }
 
 template <class T> Values::Field*
-                   Statement::BindParameter(const Local<Value> source, T pos) {
-    if (source->IsString() || source->IsRegExp()) {
-        Nan::Utf8String val(source);
-        return new Values::Text(pos, val.length(), *val);
+                   Statement::BindParameter(const Napi::Value source, T pos) {
+    if (source.IsString()) {
+        std::string val = source.As<Napi::String>().Utf8Value();
+        return new Values::Text(pos, val.length(), val.c_str());
     }
-    else if (source->IsInt32()) {
-        return new Values::Integer(pos, Nan::To<int32_t>(source).FromJust());
+    else if (OtherInstanceOf(source.As<Object>(), "RegExp")) {
+        std::string val = source.ToString().Utf8Value();
+        return new Values::Text(pos, val.length(), val.c_str());
     }
-    else if (source->IsNumber()) {
-        return new Values::Float(pos, Nan::To<double>(source).FromJust());
+    else if (source.IsNumber()) {
+        if (OtherIsInt(source.As<Napi::Number>())) {
+            return new Values::Integer(pos, source.As<Napi::Number>().Int32Value());
+        } else {
+            return new Values::Float(pos, source.As<Napi::Number>().DoubleValue());
+        }
     }
-    else if (source->IsBoolean()) {
-        return new Values::Integer(pos, Nan::To<bool>(source).FromJust() ? 1 : 0);
+    else if (source.IsBoolean()) {
+        return new Values::Integer(pos, source.As<Napi::Boolean>().Value() ? 1 : 0);
     }
-    else if (source->IsNull()) {
+    else if (source.IsNull()) {
         return new Values::Null(pos);
     }
-    else if (Buffer::HasInstance(source)) {
-        Local<Object> buffer = Nan::To<Object>(source).ToLocalChecked();
-        return new Values::Blob(pos, Buffer::Length(buffer), Buffer::Data(buffer));
+    else if (source.IsBuffer()) {
+        Napi::Buffer<char> buffer = source.As<Napi::Buffer<char>>();
+        return new Values::Blob(pos, buffer.Length(), buffer.Data());
     }
-    else if (source->IsDate()) {
-        return new Values::Float(pos, Nan::To<double>(source).FromJust());
+    else if (OtherInstanceOf(source.As<Object>(), "Date")) {
+        return new Values::Float(pos, source.ToNumber().DoubleValue());
+    }
+    else if (source.IsObject()) {
+        std::string val = source.ToString().Utf8Value();
+        return new Values::Text(pos, val.length(), val.c_str());
     }
     else {
         return NULL;
     }
 }
 
-template <class T> T* Statement::Bind(Nan::NAN_METHOD_ARGS_TYPE info, int start, int last) {
-    Nan::HandleScope scope;
+template <class T> T* Statement::Bind(const Napi::CallbackInfo& info, int start, int last) {
+    Napi::Env env = info.Env();
+    Napi::HandleScope scope(env);
 
     if (last < 0) last = info.Length();
-    Local<Function> callback;
-    if (last > start && info[last - 1]->IsFunction()) {
-        callback = Local<Function>::Cast(info[last - 1]);
+    Napi::Function callback;
+    if (last > start && info[last - 1].IsFunction()) {
+        callback = info[last - 1].As<Napi::Function>();
         last--;
     }
 
     T* baton = new T(this, callback);
 
     if (start < last) {
-        if (info[start]->IsArray()) {
-            Local<Array> array = Local<Array>::Cast(info[start]);
-            int length = array->Length();
+        if (info[start].IsArray()) {
+            Napi::Array array = info[start].As<Napi::Array>();
+            int length = array.Length();
             // Note: bind parameters start with 1.
             for (int i = 0, pos = 1; i < length; i++, pos++) {
-                baton->parameters.push_back(BindParameter(Nan::Get(array, i).ToLocalChecked(), pos));
+                baton->parameters.push_back(BindParameter((array).Get(i), pos));
             }
         }
-        else if (!info[start]->IsObject() || info[start]->IsRegExp() || info[start]->IsDate() || Buffer::HasInstance(info[start])) {
+        else if (!info[start].IsObject() || OtherInstanceOf(info[start].As<Object>(), "RegExp") || OtherInstanceOf(info[start].As<Object>(), "Date") || info[start].IsBuffer()) {
             // Parameters directly in array.
             // Note: bind parameters start with 1.
             for (int i = start, pos = 1; i < last; i++, pos++) {
                 baton->parameters.push_back(BindParameter(info[i], pos));
             }
         }
-        else if (info[start]->IsObject()) {
-            Local<Object> object = Local<Object>::Cast(info[start]);
-            Local<Array> array = Nan::GetPropertyNames(object).ToLocalChecked();
-            int length = array->Length();
+        else if (info[start].IsObject()) {
+            Napi::Object object = info[start].As<Napi::Object>();
+            Napi::Array array = object.GetPropertyNames();
+            int length = array.Length();
             for (int i = 0; i < length; i++) {
-                Local<Value> name = Nan::Get(array, i).ToLocalChecked();
+                Napi::Value name = (array).Get(i);
+                Napi::Number num = name.ToNumber();
 
-                if (name->IsInt32()) {
+                if (num.Int32Value() == num.DoubleValue()) {
                     baton->parameters.push_back(
-                        BindParameter(Nan::Get(object, name).ToLocalChecked(), Nan::To<int32_t>(name).FromJust()));
+                        BindParameter((object).Get(name), num.Int32Value()));
                 }
                 else {
-                    baton->parameters.push_back(BindParameter(Nan::Get(object, name).ToLocalChecked(),
-                        *Nan::Utf8String(name)));
+                    baton->parameters.push_back(BindParameter((object).Get(name),
+                        name.As<Napi::String>().Utf8Value().c_str()));
                 }
             }
         }
@@ -263,7 +288,7 @@ bool Statement::Bind(const Parameters & parameters) {
         Values::Field* field = *it;
 
         if (field != NULL) {
-            int pos;
+            unsigned int pos;
             if (field->index > 0) {
                 pos = field->index;
             }
@@ -305,16 +330,18 @@ bool Statement::Bind(const Parameters & parameters) {
     return true;
 }
 
-NAN_METHOD(Statement::Bind) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Bind(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     Baton* baton = stmt->Bind<Baton>(info);
     if (baton == NULL) {
-        return Nan::ThrowTypeError("Data type is not supported");
+        Napi::TypeError::New(env, "Data type is not supported").ThrowAsJavaScriptException();
+        return env.Null();
     }
     else {
         stmt->Schedule(Work_BeginBind, baton);
-        info.GetReturnValue().Set(info.This());
+        return info.This();
     }
 }
 
@@ -322,7 +349,7 @@ void Statement::Work_BeginBind(Baton* baton) {
     STATEMENT_BEGIN(Bind);
 }
 
-void Statement::Work_Bind(uv_work_t* req) {
+void Statement::Work_Bind(napi_env e, void* data) {
     STATEMENT_INIT(Baton);
 
     sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
@@ -331,20 +358,21 @@ void Statement::Work_Bind(uv_work_t* req) {
     sqlite3_mutex_leave(mtx);
 }
 
-void Statement::Work_AfterBind(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterBind(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(Baton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_OK) {
         Error(baton);
     }
     else {
         // Fire callbacks.
-        Local<Function> cb = Nan::New(baton->callback);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
-            Local<Value> argv[] = { Nan::Null() };
-            TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+        Napi::Function cb = baton->callback.Value();
+        if (!cb.IsUndefined() && cb.IsFunction()) {
+            Napi::Value argv[] = { env.Null() };
+            TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
         }
     }
 
@@ -353,16 +381,18 @@ void Statement::Work_AfterBind(uv_work_t* req) {
 
 
 
-NAN_METHOD(Statement::Get) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Get(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     Baton* baton = stmt->Bind<RowBaton>(info);
     if (baton == NULL) {
-        return Nan::ThrowError("Data type is not supported");
+        Napi::Error::New(env, "Data type is not supported").ThrowAsJavaScriptException();
+        return env.Null();
     }
     else {
         stmt->Schedule(Work_BeginGet, baton);
-        info.GetReturnValue().Set(info.This());
+        return info.This();
     }
 }
 
@@ -370,7 +400,7 @@ void Statement::Work_BeginGet(Baton* baton) {
     STATEMENT_BEGIN(Get);
 }
 
-void Statement::Work_Get(uv_work_t* req) {
+void Statement::Work_Get(napi_env e, void* data) {
     STATEMENT_INIT(RowBaton);
 
     if (stmt->status != SQLITE_DONE || baton->parameters.size()) {
@@ -394,26 +424,27 @@ void Statement::Work_Get(uv_work_t* req) {
     }
 }
 
-void Statement::Work_AfterGet(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterGet(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(RowBaton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_ROW && stmt->status != SQLITE_DONE) {
         Error(baton);
     }
     else {
         // Fire callbacks.
-        Local<Function> cb = Nan::New(baton->callback);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
+        Napi::Function cb = baton->callback.Value();
+        if (!cb.IsUndefined() && cb.IsFunction()) {
             if (stmt->status == SQLITE_ROW) {
                 // Create the result array from the data we acquired.
-                Local<Value> argv[] = { Nan::Null(), RowToJS(&baton->row) };
-                TRY_CATCH_CALL(stmt->handle(), cb, 2, argv);
+                Napi::Value argv[] = { env.Null(), RowToJS(env, &baton->row) };
+                TRY_CATCH_CALL(stmt->Value(), cb, 2, argv);
             }
             else {
-                Local<Value> argv[] = { Nan::Null() };
-                TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+                Napi::Value argv[] = { env.Null() };
+                TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
             }
         }
     }
@@ -421,16 +452,18 @@ void Statement::Work_AfterGet(uv_work_t* req) {
     STATEMENT_END();
 }
 
-NAN_METHOD(Statement::Run) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Run(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     Baton* baton = stmt->Bind<RunBaton>(info);
     if (baton == NULL) {
-        return Nan::ThrowError("Data type is not supported");
+        Napi::Error::New(env, "Data type is not supported").ThrowAsJavaScriptException();
+        return env.Null();
     }
     else {
         stmt->Schedule(Work_BeginRun, baton);
-        info.GetReturnValue().Set(info.This());
+        return info.This();
     }
 }
 
@@ -438,7 +471,7 @@ void Statement::Work_BeginRun(Baton* baton) {
     STATEMENT_BEGIN(Run);
 }
 
-void Statement::Work_Run(uv_work_t* req) {
+void Statement::Work_Run(napi_env e, void* data) {
     STATEMENT_INIT(RunBaton);
 
     sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
@@ -464,39 +497,42 @@ void Statement::Work_Run(uv_work_t* req) {
     sqlite3_mutex_leave(mtx);
 }
 
-void Statement::Work_AfterRun(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterRun(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(RunBaton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_ROW && stmt->status != SQLITE_DONE) {
         Error(baton);
     }
     else {
         // Fire callbacks.
-        Local<Function> cb = Nan::New(baton->callback);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
-            Nan::Set(stmt->handle(), Nan::New("lastID").ToLocalChecked(), Nan::New<Number>(baton->inserted_id));
-            Nan::Set(stmt->handle(), Nan::New("changes").ToLocalChecked(), Nan::New(baton->changes));
+        Napi::Function cb = baton->callback.Value();
+        if (!cb.IsUndefined() && cb.IsFunction()) {
+            (stmt->Value()).Set(Napi::String::New(env, "lastID"), Napi::Number::New(env, baton->inserted_id));
+            (stmt->Value()).Set( Napi::String::New(env, "changes"), Napi::Number::New(env, baton->changes));
 
-            Local<Value> argv[] = { Nan::Null() };
-            TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+            Napi::Value argv[] = { env.Null() };
+            TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
         }
     }
 
     STATEMENT_END();
 }
 
-NAN_METHOD(Statement::All) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::All(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     Baton* baton = stmt->Bind<RowsBaton>(info);
     if (baton == NULL) {
-        return Nan::ThrowError("Data type is not supported");
+        Napi::Error::New(env, "Data type is not supported").ThrowAsJavaScriptException();
+        return env.Null();
     }
     else {
         stmt->Schedule(Work_BeginAll, baton);
-        info.GetReturnValue().Set(info.This());
+        return info.This();
     }
 }
 
@@ -504,7 +540,7 @@ void Statement::Work_BeginAll(Baton* baton) {
     STATEMENT_BEGIN(All);
 }
 
-void Statement::Work_All(uv_work_t* req) {
+void Statement::Work_All(napi_env e, void* data) {
     STATEMENT_INIT(RowsBaton);
 
     sqlite3_mutex* mtx = sqlite3_db_mutex(stmt->db->_handle);
@@ -530,38 +566,39 @@ void Statement::Work_All(uv_work_t* req) {
     sqlite3_mutex_leave(mtx);
 }
 
-void Statement::Work_AfterAll(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterAll(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(RowsBaton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_DONE) {
         Error(baton);
     }
     else {
         // Fire callbacks.
-        Local<Function> cb = Nan::New(baton->callback);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
+        Napi::Function cb = baton->callback.Value();
+        if (!cb.IsUndefined() && cb.IsFunction()) {
             if (baton->rows.size()) {
                 // Create the result array from the data we acquired.
-                Local<Array> result(Nan::New<Array>(baton->rows.size()));
+                Napi::Array result(Napi::Array::New(env, baton->rows.size()));
                 Rows::const_iterator it = baton->rows.begin();
                 Rows::const_iterator end = baton->rows.end();
                 for (int i = 0; it < end; ++it, i++) {
-                    Nan::Set(result, i, RowToJS(*it));
+                    (result).Set(i, RowToJS(env,*it));
                     delete *it;
                 }
 
-                Local<Value> argv[] = { Nan::Null(), result };
-                TRY_CATCH_CALL(stmt->handle(), cb, 2, argv);
+                Napi::Value argv[] = { env.Null(), result };
+                TRY_CATCH_CALL(stmt->Value(), cb, 2, argv);
             }
             else {
                 // There were no result rows.
-                Local<Value> argv[] = {
-                    Nan::Null(),
-                    Nan::New<Array>(0)
+                Napi::Value argv[] = {
+                    env.Null(),
+                    Napi::Array::New(env, 0)
                 };
-                TRY_CATCH_CALL(stmt->handle(), cb, 2, argv);
+                TRY_CATCH_CALL(stmt->Value(), cb, 2, argv);
             }
         }
     }
@@ -569,24 +606,26 @@ void Statement::Work_AfterAll(uv_work_t* req) {
     STATEMENT_END();
 }
 
-NAN_METHOD(Statement::Each) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Each(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     int last = info.Length();
 
-    Local<Function> completed;
-    if (last >= 2 && info[last - 1]->IsFunction() && info[last - 2]->IsFunction()) {
-        completed = Local<Function>::Cast(info[--last]);
+    Napi::Function completed;
+    if (last >= 2 && info[last - 1].IsFunction() && info[last - 2].IsFunction()) {
+        completed = info[--last].As<Napi::Function>();
     }
 
     EachBaton* baton = stmt->Bind<EachBaton>(info, 0, last);
     if (baton == NULL) {
-        return Nan::ThrowError("Data type is not supported");
+        Napi::Error::New(env, "Data type is not supported").ThrowAsJavaScriptException();
+        return env.Null();
     }
     else {
-        baton->completed.Reset(completed);
+        baton->completed.Reset(completed, 1);
         stmt->Schedule(Work_BeginEach, baton);
-        info.GetReturnValue().Set(info.This());
+        return info.This();
     }
 }
 
@@ -595,13 +634,13 @@ void Statement::Work_BeginEach(Baton* baton) {
     // the event loop. This prevents dangling events.
     EachBaton* each_baton = static_cast<EachBaton*>(baton);
     each_baton->async = new Async(each_baton->stmt, reinterpret_cast<uv_async_cb>(AsyncEach));
-    each_baton->async->item_cb.Reset(each_baton->callback);
-    each_baton->async->completed_cb.Reset(each_baton->completed);
+    each_baton->async->item_cb.Reset(each_baton->callback.Value(), 1);
+    each_baton->async->completed_cb.Reset(each_baton->completed.Value(), 1);
 
     STATEMENT_BEGIN(Each);
 }
 
-void Statement::Work_Each(uv_work_t* req) {
+void Statement::Work_Each(napi_env e, void* data) {
     STATEMENT_INIT(EachBaton);
 
     Async* async = baton->async;
@@ -651,11 +690,12 @@ void Statement::CloseCallback(uv_handle_t* handle) {
     delete async;
 }
 
-void Statement::AsyncEach(uv_async_t* handle, int status) {
-    Nan::HandleScope scope;
-
+void Statement::AsyncEach(uv_async_t* handle) {
     Async* async = static_cast<Async*>(handle->data);
 
+    Napi::Env env = async->stmt->Env();
+    Napi::HandleScope scope(env);
+
     while (true) {
         // Get the contents out of the data cache for us to process in the JS callback.
         Rows rows;
@@ -667,41 +707,42 @@ void Statement::AsyncEach(uv_async_t* handle, int status) {
             break;
         }
 
-        Local<Function> cb = Nan::New(async->item_cb);
-        if (!cb.IsEmpty() && cb->IsFunction()) {
-            Local<Value> argv[2];
-            argv[0] = Nan::Null();
+        Napi::Function cb = async->item_cb.Value();
+        if (!cb.IsUndefined() && cb.IsFunction()) {
+            Napi::Value argv[2];
+            argv[0] = env.Null();
 
             Rows::const_iterator it = rows.begin();
             Rows::const_iterator end = rows.end();
             for (int i = 0; it < end; ++it, i++) {
-                argv[1] = RowToJS(*it);
+                argv[1] = RowToJS(env,*it);
                 async->retrieved++;
-                TRY_CATCH_CALL(async->stmt->handle(), cb, 2, argv);
+                TRY_CATCH_CALL(async->stmt->Value(), cb, 2, argv);
                 delete *it;
             }
         }
     }
 
-    Local<Function> cb = Nan::New(async->completed_cb);
+    Napi::Function cb = async->completed_cb.Value();
     if (async->completed) {
         if (!cb.IsEmpty() &&
-                cb->IsFunction()) {
-            Local<Value> argv[] = {
-                Nan::Null(),
-                Nan::New(async->retrieved)
+                cb.IsFunction()) {
+            Napi::Value argv[] = {
+                env.Null(),
+                Napi::Number::New(env, async->retrieved)
             };
-            TRY_CATCH_CALL(async->stmt->handle(), cb, 2, argv);
+            TRY_CATCH_CALL(async->stmt->Value(), cb, 2, argv);
         }
         uv_close(reinterpret_cast<uv_handle_t*>(handle), CloseCallback);
     }
 }
 
-void Statement::Work_AfterEach(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterEach(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(EachBaton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     if (stmt->status != SQLITE_DONE) {
         Error(baton);
     }
@@ -709,74 +750,76 @@ void Statement::Work_AfterEach(uv_work_t* req) {
     STATEMENT_END();
 }
 
-NAN_METHOD(Statement::Reset) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Reset(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
 
     OPTIONAL_ARGUMENT_FUNCTION(0, callback);
 
     Baton* baton = new Baton(stmt, callback);
     stmt->Schedule(Work_BeginReset, baton);
 
-    info.GetReturnValue().Set(info.This());
+    return info.This();
 }
 
 void Statement::Work_BeginReset(Baton* baton) {
     STATEMENT_BEGIN(Reset);
 }
 
-void Statement::Work_Reset(uv_work_t* req) {
+void Statement::Work_Reset(napi_env e, void* data) {
     STATEMENT_INIT(Baton);
 
     sqlite3_reset(stmt->_handle);
     stmt->status = SQLITE_OK;
 }
 
-void Statement::Work_AfterReset(uv_work_t* req) {
-    Nan::HandleScope scope;
-
+void Statement::Work_AfterReset(napi_env e, napi_status status, void* data) {
     STATEMENT_INIT(Baton);
 
+    Napi::Env env = stmt->Env();
+    Napi::HandleScope scope(env);
+
     // Fire callbacks.
-    Local<Function> cb = Nan::New(baton->callback);
-    if (!cb.IsEmpty() && cb->IsFunction()) {
-        Local<Value> argv[] = { Nan::Null() };
-        TRY_CATCH_CALL(stmt->handle(), cb, 1, argv);
+    Napi::Function cb = baton->callback.Value();
+    if (!cb.IsUndefined() && cb.IsFunction()) {
+        Napi::Value argv[] = { env.Null() };
+        TRY_CATCH_CALL(stmt->Value(), cb, 1, argv);
     }
 
     STATEMENT_END();
 }
 
-Local<Object> Statement::RowToJS(Row* row) {
-    Nan::EscapableHandleScope scope;
+Napi::Value Statement::RowToJS(Napi::Env env, Row* row) {
+    Napi::EscapableHandleScope scope(env);
 
-    Local<Object> result = Nan::New<Object>();
+    Napi::Object result = Napi::Object::New(env);
 
     Row::const_iterator it = row->begin();
     Row::const_iterator end = row->end();
     for (int i = 0; it < end; ++it, i++) {
         Values::Field* field = *it;
 
-        Local<Value> value;
+        Napi::Value value;
 
         switch (field->type) {
             case SQLITE_INTEGER: {
-                value = Nan::New<Number>(((Values::Integer*)field)->value);
+                value = Napi::Number::New(env, ((Values::Integer*)field)->value);
             } break;
             case SQLITE_FLOAT: {
-                value = Nan::New<Number>(((Values::Float*)field)->value);
+                value = Napi::Number::New(env, ((Values::Float*)field)->value);
             } break;
             case SQLITE_TEXT: {
-                value = Nan::New<String>(((Values::Text*)field)->value.c_str(), ((Values::Text*)field)->value.size()).ToLocalChecked();
+                value = Napi::String::New(env, ((Values::Text*)field)->value.c_str(), ((Values::Text*)field)->value.size());
             } break;
             case SQLITE_BLOB: {
-                value = Nan::CopyBuffer(((Values::Blob*)field)->value, ((Values::Blob*)field)->length).ToLocalChecked();
+                value = Napi::Buffer<char>::Copy(env, ((Values::Blob*)field)->value, ((Values::Blob*)field)->length);
             } break;
             case SQLITE_NULL: {
-                value = Nan::Null();
+                value = env.Null();
             } break;
         }
 
-        Nan::Set(result, Nan::New(field->name.c_str()).ToLocalChecked(), value);
+        (result).Set(Napi::String::New(env, field->name.c_str()), value);
 
         DELETE_FIELD(field);
     }
@@ -816,31 +859,33 @@ void Statement::GetRow(Row* row, sqlite3_stmt* stmt) {
     }
 }
 
-NAN_METHOD(Statement::Finalize) {
-    Statement* stmt = Nan::ObjectWrap::Unwrap<Statement>(info.This());
+Napi::Value Statement::Finalize_(const Napi::CallbackInfo& info) {
+    Napi::Env env = info.Env();
+    Statement* stmt = this;
     OPTIONAL_ARGUMENT_FUNCTION(0, callback);
 
     Baton* baton = new Baton(stmt, callback);
-    stmt->Schedule(Finalize, baton);
+    stmt->Schedule(Finalize_, baton);
 
-    info.GetReturnValue().Set(stmt->db->handle());
+    return stmt->db->Value();
 }
 
-void Statement::Finalize(Baton* baton) {
-    Nan::HandleScope scope;
+void Statement::Finalize_(Baton* baton) {
+    Napi::Env env = baton->stmt->Env();
+    Napi::HandleScope scope(env);
 
-    baton->stmt->Finalize();
+    baton->stmt->Finalize_();
 
     // Fire callback in case there was one.
-    Local<Function> cb = Nan::New(baton->callback);
-    if (!cb.IsEmpty() && cb->IsFunction()) {
-        TRY_CATCH_CALL(baton->stmt->handle(), cb, 0, NULL);
+    Napi::Function cb = baton->callback.Value();
+    if (!cb.IsUndefined() && cb.IsFunction()) {
+        TRY_CATCH_CALL(baton->stmt->Value(), cb, 0, NULL);
     }
 
     delete baton;
 }
 
-void Statement::Finalize() {
+void Statement::Finalize_() {
     assert(!finalized);
     finalized = true;
     CleanQueue();
@@ -852,13 +897,14 @@ void Statement::Finalize() {
 }
 
 void Statement::CleanQueue() {
-    Nan::HandleScope scope;
+    Napi::Env env = this->Env();
+    Napi::HandleScope scope(env);
 
     if (prepared && !queue.empty()) {
         // This statement has already been prepared and is now finalized.
         // Fire error for all remaining items in the queue.
-        EXCEPTION("Statement is already finalized", SQLITE_MISUSE, exception);
-        Local<Value> argv[] = { exception };
+        EXCEPTION(Napi::String::New(env, "Statement is already finalized"), SQLITE_MISUSE, exception);
+        Napi::Value argv[] = { exception };
         bool called = false;
 
         // Clear out the queue so that this object can get GC'ed.
@@ -866,11 +912,11 @@ void Statement::CleanQueue() {
             Call* call = queue.front();
             queue.pop();
 
-            Local<Function> cb = Nan::New(call->baton->callback);
+            Napi::Function cb = call->baton->callback.Value();
 
             if (prepared && !cb.IsEmpty() &&
-                cb->IsFunction()) {
-                TRY_CATCH_CALL(handle(), cb, 1, argv);
+                cb.IsFunction()) {
+                TRY_CATCH_CALL(Value(), cb, 1, argv);
                 called = true;
             }
 
@@ -883,8 +929,8 @@ void Statement::CleanQueue() {
         // When we couldn't call a callback function, emit an error on the
         // Statement object.
         if (!called) {
-            Local<Value> info[] = { Nan::New("error").ToLocalChecked(), exception };
-            EMIT_EVENT(handle(), 2, info);
+            Napi::Value info[] = { Napi::String::New(env, "error"), exception };
+            EMIT_EVENT(Value(), 2, info);
         }
     }
     else while (!queue.empty()) {
